@@ -12,4 +12,4 @@ from tensorfn.config.config import (
 from tensorfn.config.optimizer import Optimizer, make_optimizer
 from tensorfn.config.lr_scheduler import Scheduler
 from tensorfn.config.data import DataLoader, make_dataloader
-from tensorfn.config.checker import Checker
+from tensorfn.config.checker import Checker, Checker2
