@@ -66,13 +66,16 @@ def get_logger(distributed_rank=None, *, mode="rich", name="main", abbrev_name=N
     Returns:
         logging.Logger: a logger
     """
-
     if distributed_rank is None:
         distributed_rank = dist.get_rank()
 
     logging.setLogRecordFactory(wrap_log_record_factory(logging.getLogRecordFactory()))
 
     logger = logging.getLogger(name)
+    
+    if logger.handlers:
+        return logger
+    
     logger.setLevel(logging.DEBUG)
     logger.propagate = False
 
